epan_range_h_types_cdef = """

/* range.h
 * Range routines
 *
 * Dick Gooris <gooris@lucent.com>
 * Ulf Lamping <ulf.lamping@web.de>
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/** @file
 * Range strings a variant of value_strings
 */

/**@todo where's the best place for these? */
#define MAX_SCTP_PORT 65535
#define MAX_TCP_PORT 65535
#define MAX_UDP_PORT 65535
#define MAX_DCCP_PORT 65535

typedef struct range_admin_tag {
    guint32 low;
    guint32 high;
} range_admin_t;
// FIXME: Later - #define RANGE_ADMIN_T_INITIALIZER { 0, 0 }

/** user specified range(s) */
typedef struct epan_range {
    guint           nranges;   /**< number of entries in ranges */
    range_admin_t   ranges[1]; /**< variable-length array */
} range_t;

/**
 * Return value from range_convert_str().
 */
typedef enum {
    CVT_NO_ERROR,
    CVT_SYNTAX_ERROR,
    CVT_NUMBER_TOO_BIG
} convert_ret_t;

"""

epan_range_h_funcs_cdef = """
WS_DLL_PUBLIC range_t *range_empty(wmem_allocator_t *scope);


/*** Converts a range string to a fast comparable array of ranges.
 * This function allocates a range_t large enough to hold the number
 * of ranges specified, and fills the array range->ranges containing
 * low and high values with the number of ranges being range->nranges.
 * After having called this function, the function value_is_in_range()
 * determines whether a given number is within the range or not.<BR>
 * In case of a single number, we make a range where low is equal to high.
 * We take care on wrongly entered ranges; opposite order will be taken
 * care of.
 *
 * The following syntax is accepted :
 *
 *   1-20,30-40     Range from 1 to 20, and packets 30 to 40
 *   -20,30         Range from 1 to 20, and packet 30
 *   20,30,40-      20, 30, and the range from 40 to the end
 *   20-10,30-25    Range from 10 to 20, and from 25 to 30
 *   -              All values
 * @param scope memory scope for the range
 * @param range the range
 * @param es points to the string to be converted.
 * @param max_value specifies the maximum value in a range.
 * @return convert_ret_t
 */
WS_DLL_PUBLIC convert_ret_t range_convert_str(wmem_allocator_t *scope, range_t **range, const gchar *es,
    guint32 max_value);

WS_DLL_PUBLIC convert_ret_t range_convert_str_work(wmem_allocator_t *scope, range_t **range, const gchar *es,
    guint32 max_value, gboolean err_on_max);

/** This function returns TRUE if a given value is within one of the ranges
 * stored in the ranges array.
 * @param range the range
 * @param val the value to check
 * @return TRUE if the value is in range
 */
WS_DLL_PUBLIC gboolean value_is_in_range(range_t *range, guint32 val);

/** This function returns TRUE if val has successfully been added to
 * a range.  This may extend an existing range or create a new one
 * @param scope memory scope of range (in case of reallocation)
 * @param range to add value
 * @param val value to add to range
 * @return TRUE if the value is successsfully added to range
 */
WS_DLL_PUBLIC gboolean range_add_value(wmem_allocator_t *scope, range_t **range, guint32 val);

/** This function returns TRUE if val has successfully been removed from
 * a range.  This may remove an existing range.
 * @param scope memory scope of range (in case of reallocation)
 * @param range to remove value
 * @param val value to remove within range
 * @return TRUE if the value is successsfully removed to range
 */
WS_DLL_PUBLIC gboolean range_remove_value(wmem_allocator_t *scope, range_t **range, guint32 val);

/** This function returns TRUE if the two given range_t's are equal.
 * @param a first range
 * @param b second range
 * @return TRUE if the value is in range
 */
WS_DLL_PUBLIC gboolean ranges_are_equal(range_t *a, range_t *b);

/** This function calls the provided callback function for each value in
 * in the range. Takes a pointer argument, which is passed to the
 * callback, along with the value in the range.
 * @param range the range
 * @param callback the callback function
 * @param ptr pointer passed to the callback
 */
WS_DLL_PUBLIC void range_foreach(range_t *range, void (*callback)(guint32 val, gpointer ptr), gpointer ptr);

/**
 * This function converts a range_t to a (wmem_alloc()-allocated) string.
 */
WS_DLL_PUBLIC char *range_convert_range(wmem_allocator_t *scope, const range_t *range);

/**
 * Create a (wmem-alloc()ed) copy of a range
 * @param scope memory scope for the copied range
 * @param src the range to copy
 * @return ep allocated copy of the range
 */
WS_DLL_PUBLIC range_t *range_copy(wmem_allocator_t *scope, range_t *src);



"""



libc_typedefs_h_cdef = """

typedef unsigned int u_int;
typedef unsigned short u_short;
typedef unsigned char u_char;

// From sys/time.h -- __kernel_time_t is long
struct timeval {
    long tv_sec;
    long tv_usec;

};

"""



"""
wrapper library to actual wireshark bindings. Should not be used directly.

As support for more 'wireshark' features gets added, the support will be added
in the _wrapper and other APIs (like WishpyDissector) will make use of that
support. This abstracts out Wireshark 2.6 and Wireshark 3.2 specific
API calls.
"""

import os
import binascii
import warnings

try:
    from .epan2_ext import lib as epan_lib
    from .epan2_ext import ffi as epan_ffi
    _epan_version = (2, 6)
except ImportError:
    try:
        from .epan3_ext import lib as epan_lib
        from .epan3_ext import ffi as epan_ffi
        _epan_version = (3, 2)
    except ImportError:
        warnings.warn("Epan Library Extensions Not Found.")
        raise EpanLoadError

class EpanLoadError(Exception):
    pass

_nstime_empty = epan_ffi.new('nstime_t *');
@epan_ffi.callback('const nstime_t *(*)(struct packet_provider_data *prov, guint32 frame_num)')
def _wishpy_get_ts(prov, frame_num):
    return _nstime_empty


_wishpy_provider_funcs = epan_ffi.new('struct packet_provider_funcs *',
        [_wishpy_get_ts, epan_ffi.NULL, epan_ffi.NULL, epan_ffi.NULL])

def _epan_init_v2():

    epan_lib.init_process_policies()
    null_register_cb = epan_ffi.cast('register_cb', epan_ffi.NULL)
    register_protocols_handle = epan_ffi.callback('void (*)(register_cb, gpointer)',
            epan_lib.register_all_protocols)
    register_handoffs_handle = epan_ffi.callback('void (*)(register_cb, gpointer)',
            epan_lib.register_all_protocol_handoffs)
    result = epan_lib.epan_init(
            register_protocols_handle,
            register_handoffs_handle,
            null_register_cb, epan_ffi.NULL)
    if result:
        epan_lib.epan_load_settings()

    return result

def _epan_init_v3():
    """
    Performs initialzation of the `epan` library. Calls
    1. `init_process_policies` (Required)
    2. `epan_init` (Required)
    3. `epan_load_settings` (Required)

    Returns True/False based upon result of operation.
    """

    epan_lib.init_process_policies()
    null_register_cb = epan_ffi.cast('register_cb', epan_ffi.NULL)
    result = epan_lib.epan_init(
            null_register_cb, epan_ffi.NULL, True)
    if result:
        epan_lib.epan_load_settings()

    return result

def _epan_perform_one_packet_dissection_v2(frames, hdr, packet_data, cb_func):
    """Performs dissection of a single packet using v2.6
    """

    # FIXME : hardcoded
    wth_file_type = 1
    total_bytes = 0

    # FIXME: epan_session and epan_dissect_obj to be set by caller? So that
    # We don't free on every data.
    epan_session = epan_lib.epan_new(epan_ffi.NULL, _wishpy_provider_funcs)

    epan_dissect_obj = epan_lib.epan_dissect_new(epan_session, True, True)

    # Read stuff from `hdr` for us
    # TODO: Timestamps
    packet_len = hdr[0].len
    packet_capture_len = hdr[0].caplen

    # Allocate enough space to 'copy'
    count = packet_capture_len
    if packet_capture_len > 1024:
        count = 4096
    if packet_capture_len > 4096:
        count = 16384
    if packet_capture_len > 16384:
        count = 65536
    alloc_str = 'guint8[{:d}]'.format(count)

    buf_ptr = epan_ffi.new(alloc_str)
    # Copy from the data `pakcet_data` to us
    epan_ffi.memmove(buf_ptr, packet_data, packet_capture_len)

    # Initialize Frame Data now
    elapsed_time_ptr = epan_ffi.new('nstime_t *')
    frame_data_ptr = epan_ffi.new('frame_data *')

    frame_data_ref = epan_ffi.new('frame_data **')
    frame_data_ref[0] = epan_ffi.NULL

    offset = epan_ffi.new('gint64 *')
    cum_bytes = epan_ffi.new('guint32 *')

    # Go ahead fill-up the `rec`
    rec = epan_ffi.new('wtap_rec *')
    epan_lib.wtap_rec_init(rec)
    rec[0].rec_header.packet_header.len = packet_len
    rec[0].rec_header.packet_header.caplen = packet_capture_len
    rec[0].rec_header.packet_header.pkt_encap = 1 # FIXME: Hard coded
    rec[0].ts.secs = hdr[0].ts.tv_sec
    rec[0].ts.nsecs = hdr[0].ts.tv_usec * 1000 # Asumes usec precision FIXME
    rec[0].presence_flags = epan_lib.WTAP_HAS_TS | epan_lib.WTAP_HAS_CAP_LEN

    epan_lib.frame_data_init(frame_data_ptr, frames, rec,
            offset[0], cum_bytes[0])

    # FIXME: Look at properly using `frame_data_ref`
    epan_lib.frame_data_set_before_dissect(frame_data_ptr,
            elapsed_time_ptr, frame_data_ref, epan_ffi.NULL)

    # Not sure what this is - Just copied from `tshark` sources
    epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(epan_dissect_obj)

    ## Do actual dissection here.
    # Get buffer and tvbuff first and then run dissector
    tvb_ptr = epan_lib.tvb_new_real_data(buf_ptr, packet_len, packet_capture_len)
    epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type, rec,
            tvb_ptr, frame_data_ptr, epan_ffi.NULL)

    dissected = cb_func(epan_dissect_obj)

    # Reset the frame data and dissector object
    epan_lib.frame_data_set_after_dissect(frame_data_ptr, cum_bytes)
    epan_lib.epan_dissect_reset(epan_dissect_obj)

    epan_lib.epan_dissect_free(epan_dissect_obj)
    epan_lib.epan_free(epan_session)

    return dissected

def _epan_perform_dissection_v2(wth, wth_file_type, cb_func, count=0, skip=-1):
    """
    Performs dissection of the file bound to `wth`
    """

    epan_session = epan_lib.epan_new(epan_ffi.NULL, _wishpy_provider_funcs)

    # TODO: make proper `epan_session` for us
    epan_dissect_obj = epan_lib.epan_dissect_new(epan_session, True, True)

    offset = epan_ffi.new('gint64 *')
    frame_data_ref = epan_ffi.new('frame_data **')
    elapsed_time_ptr = epan_ffi.new('nstime_t *')

    err = epan_ffi.new('int *')
    err_str = epan_ffi.new("gchar **")

    frame_data_ref[0] = epan_ffi.NULL
    frame_data_ptr = epan_ffi.new('frame_data *')
    cum_bytes = epan_ffi.new('guint32 *')
    processed = 0
    total_bytes = 0
    while True:
        result = epan_lib.wtap_read(wth, err, err_str, offset)

        if result == True:
            processed += 1
            rec = epan_lib.wtap_get_rec(wth)
            buf_ptr = epan_lib.wtap_get_buf_ptr(wth)

            pkt_len = rec.rec_header.packet_header.len
            pkt_reported_len = rec.rec_header.packet_header.caplen

            epan_lib.frame_data_init(frame_data_ptr, processed, rec,
                    offset[0], cum_bytes[0])

            total_bytes += pkt_reported_len
            cum_bytes[0] = total_bytes

            # FIXME: Look at properly using `frame_data_ref`
            epan_lib.frame_data_set_before_dissect(frame_data_ptr,
                    elapsed_time_ptr, frame_data_ref, epan_ffi.NULL)

            # Not sure what this is - Just copied from `tshark` sources
            epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
                    epan_dissect_obj)

            ## Do actual dissection here.
            # Get buffer and tvbuff first and then run dissector
            tvb_ptr = epan_lib.tvb_new_real_data(buf_ptr, pkt_len,
                    pkt_reported_len)

            epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type,
                    rec, tvb_ptr, frame_data_ptr, epan_ffi.NULL)

            dissected = cb_func(epan_dissect_obj)

            # Reset the frame data and dissector object
            epan_lib.frame_data_set_after_dissect(frame_data_ptr, cum_bytes)
            epan_lib.epan_dissect_reset(epan_dissect_obj)

            processed += 1

            yield dissected

            if processed == count:
                break

        else:
            break

    epan_lib.epan_dissect_free(epan_dissect_obj)
    epan_lib.epan_free(epan_session)

def _epan_perform_one_packet_dissection_v3(frames, hdr, packet_data, cb_func):
    """Performs a single packet dissection.
    """

    # FIXME : hardcoded
    wth_file_type = 1
    total_bytes = 0

    # FIXME: epan_session and epan_dissect_obj to be set by caller? So that
    # We don't free on every data.
    epan_session = epan_lib.epan_new(epan_ffi.NULL, _wishpy_provider_funcs)

    epan_dissect_obj = epan_lib.epan_dissect_new(epan_session, True, True)

    # Read stuff from `hdr` for us
    # TODO: Timestamps
    packet_len = hdr[0].len
    packet_capture_len = hdr[0].caplen

    # Allocate enough space to 'copy'
    count = packet_capture_len
    if packet_capture_len > 1024:
        count = 4096
    if packet_capture_len > 4096:
        count = 16384
    if packet_capture_len > 16384:
        count = 65536

    # Copy from the data `packet_data` to us
    # We've to oblige to `wireshark` way of doing it hence the heavy lifting
    buf = epan_ffi.new('Buffer *')
    epan_lib.ws_buffer_init(buf, count) # FIXME : Should do with proper length
    start_ptr = buf[0].data + buf[0].start
    epan_ffi.memmove(start_ptr, packet_data, packet_capture_len)

    # Initialize Frame Data now
    frame_data_ptr = epan_ffi.new('frame_data *')

    rec = epan_ffi.new('wtap_rec *')
    epan_lib.wtap_rec_init(rec)
    rec[0].rec_header.packet_header.len = packet_len
    rec[0].rec_header.packet_header.caplen = packet_capture_len
    rec[0].rec_header.packet_header.pkt_encap = 1 # FIXME: Hard coded
    rec[0].ts.secs = hdr[0].ts.tv_sec
    rec[0].ts.nsecs = hdr[0].ts.tv_usec * 1000 # Asumes usec precision FIXME
    rec[0].presence_flags = epan_lib.WTAP_HAS_TS | epan_lib.WTAP_HAS_CAP_LEN

    offset = epan_ffi.new('gint64 *')
    cum_bytes = epan_ffi.new('guint32 *')
    epan_lib.frame_data_init(frame_data_ptr, frames, rec, offset[0], cum_bytes[0])

    total_bytes += packet_capture_len
    cum_bytes[0] = total_bytes

    # FIXME: Look at properly using `frame_data_ref`
    elapsed_time_ptr = epan_ffi.new('nstime_t *')
    frame_data_ref = epan_ffi.new('frame_data **')
    frame_data_ref[0] = epan_ffi.NULL
    epan_lib.frame_data_set_before_dissect(frame_data_ptr,
            elapsed_time_ptr, frame_data_ref, epan_ffi.NULL)

    # Not sure what this is - Just copied from `tshark` sources
    epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
            epan_dissect_obj)

    ## Ready to do `dissection` here.
    # Get buffer and tvbuff first and then run dissector
    tvb_ptr = epan_lib.tvb_new_real_data(buf[0].data, packet_len,
            packet_capture_len)

    epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type,
            rec, tvb_ptr, frame_data_ptr, epan_ffi.NULL)

    dissected = cb_func(epan_dissect_obj)

    # Reset the frame data and dissector object
    epan_lib.frame_data_set_after_dissect(frame_data_ptr, cum_bytes)
    epan_lib.epan_dissect_reset(epan_dissect_obj)

    epan_lib.epan_dissect_free(epan_dissect_obj)
    epan_lib.epan_free(epan_session)

    return dissected


def _epan_perform_dissection_v3(wth, wth_file_type, cb_func, count=0, skip=-1):
    """
    Performs dissection of the file bound to `wth`. If Non-zero positive count
    is specified, performs dissection of up to `count` packets
    """

    epan_session = epan_lib.epan_new(epan_ffi.NULL, _wishpy_provider_funcs)

    # TODO: make proper `epan_session` for us
    epan_dissect_obj = epan_lib.epan_dissect_new(epan_session, True, True)

    offset = epan_ffi.new('gint64 *')
    frame_data_ref = epan_ffi.new('frame_data **')
    elapsed_time_ptr = epan_ffi.new('nstime_t *')

    buf = epan_ffi.new('Buffer *')
    epan_lib.ws_buffer_init(buf, 1514) # FIXME : Should do with proper length
    rec = epan_ffi.new('wtap_rec *')
    epan_lib.wtap_rec_init(rec)
    err = epan_ffi.new('int *')
    err_str = epan_ffi.new("gchar **")

    frame_data_ref[0] = epan_ffi.NULL
    frame_data_ptr = epan_ffi.new('frame_data *')
    cum_bytes = epan_ffi.new('guint32 *')
    processed = 0
    total_bytes = 0
    skipped = 0
    while True:
        result = epan_lib.wtap_read(wth, rec, buf, err, err_str, offset)

        if result == True:

            pkt_len = rec[0].rec_header.packet_header.len
            pkt_reported_len = rec[0].rec_header.packet_header.caplen

            epan_lib.frame_data_init(frame_data_ptr, processed, rec,
                    offset[0], cum_bytes[0])

            total_bytes += pkt_reported_len
            cum_bytes[0] = total_bytes

            # FIXME: Look at properly using `frame_data_ref`
            epan_lib.frame_data_set_before_dissect(frame_data_ptr,
                    elapsed_time_ptr, frame_data_ref, epan_ffi.NULL)

            # Not sure what this is - Just copied from `tshark` sources
            epan_lib.prime_epan_dissect_with_postdissector_wanted_hfids(
                    epan_dissect_obj)

            ## Do actual dissection here.
            # Get buffer and tvbuff first and then run dissector
            tvb_ptr = epan_lib.tvb_new_real_data(buf[0].data, pkt_len,
                    pkt_reported_len)

            epan_lib.epan_dissect_run(epan_dissect_obj, wth_file_type,
                    rec, tvb_ptr, frame_data_ptr, epan_ffi.NULL)

            if skipped < skip:
                skipped += 1
                continue

            dissected = cb_func(epan_dissect_obj)

            # Reset the frame data and dissector object
            epan_lib.frame_data_set_after_dissect(frame_data_ptr, cum_bytes)
            epan_lib.epan_dissect_reset(epan_dissect_obj)

            processed += 1

            yield dissected

            if processed == count:
                break

        else:
            break

    epan_lib.epan_dissect_free(epan_dissect_obj)
    epan_lib.epan_free(epan_session)

    return processed

def wtap_open_file_offline(filepath):
    """
    Opens a file given by filepath  using `wtap_open_offline` and returns a tuple containing
    Handle and file type.
    """

    if not os.path.exists(filepath):
        return None, None

    # Open a wtap file
    err = epan_ffi.new('int *')
    err_str = epan_ffi.new("gchar *[256]")
    filename = filepath
    open_type = epan_lib.WTAP_TYPE_AUTO
    wth = epan_lib.wtap_open_offline(filename.encode(), open_type, err, err_str, False)
    if wth is epan_ffi.NULL:
        return None, None

    wtap_file_type = epan_lib.wtap_file_type_subtype(wth)
    return wth, wtap_file_type

wtap_close = epan_lib.wtap_close
epan_cleanup = epan_lib.epan_cleanup


if _epan_version == (2,6):
    epan_lib_init = _epan_init_v2
    epan_perform_dissection = _epan_perform_dissection_v2
    epan_perform_one_packet_dissection = _epan_perform_one_packet_dissection_v2
else:
    epan_lib_init = _epan_init_v3
    epan_perform_dissection = _epan_perform_dissection_v3
    epan_perform_one_packet_dissection = _epan_perform_one_packet_dissection_v3

def perform_epan_wtap_init():

    # FIXME: True / False to be decided
    epan_lib.wtap_init(False)

    return epan_lib_init()

def perform_epan_wtap_cleanup():

    epan_lib.epan_cleanup()
    epan_lib.wtap_cleanup()



from cffi import FFI
from cffi import PkgConfigError


from .typedefs_h import libc_typedefs_h_cdef

from .bpf_h import libpcap_bpf_h_cdef

from .pcap_h import (
        libpcap_pcap_h_all_cdef,
        libpcap_pcap_h_nowin_cdef,
        libpcap_pcap_h_linux_cdef)

libpcap_ffi = FFI()

libpcap_ffi.cdef(libc_typedefs_h_cdef)
libpcap_ffi.cdef(libpcap_bpf_h_cdef)

libpcap_ffi.cdef(libpcap_pcap_h_all_cdef)
libpcap_ffi.cdef(libpcap_pcap_h_nowin_cdef)
libpcap_ffi.cdef(libpcap_pcap_h_linux_cdef)


_pkg_name = 'wishpy.libpcap.lib.libpcap_ext'
_pkgconfig_libs = ['libpcap']

_sources = '''
    #include <pcap/pcap.h>
    '''

_extra_compile_args = ['-I/usr/local/include']
_extra_link_args = ['-L/usr/local/lib', '-lpcap']

try:
    libpcap_ffi.set_source_pkgconfig(_pkg_name, _pkgconfig_libs, _sources)
except PkgConfigError:
    libpcap_ffi.set_source(
            _pkg_name,
            _sources,
            extra_compile_args=_extra_compile_args,
            extra_link_args=_extra_link_args)

if __name__ == '__main__':
    pass



from wishpy.libpcap.src.pcap import (
        libpcap_ffi,
        _sources,
        _extra_link_args,
        _extra_compile_args)



if __name__ == '__main__':

    libpcap_lib = libpcap_ffi.verify(_sources,
            extra_link_args=_extra_link_args,
            extra_compile_args=_extra_compile_args)

    print(libpcap_lib)



"""
pcapy compatible API

This module provides `pcapy` (https://github.com/helpsystems/pcapy)
"""
import warnings
import sys
import ipaddress

from .libpcap_ext import lib as _pcap_lib
from .libpcap_ext import ffi as _pcap_ffi


class PcapError(Exception):
    pass


def open_live(device, snaplen, promisc, to_ms):
    """Returns a live capture class Reader for a given device

    Internally performs `pcap_open_live` and `pcap_lookup_net`
    Args:
        device: string - Device to open
        snaplen: int - Requested snap length.
        promis: bool - Requested promiscuous mode.
        to_ms : int - Timeout in miliseconds.
    """
    err_buf = _pcap_ffi.new('char [256]')
    net = _pcap_ffi.new('bpf_u_int32 *')
    mask = _pcap_ffi.new('bpf_u_int32 *')
    error = _pcap_lib.pcap_lookupnet(device.encode(), net, mask, err_buf)
    if error != 0:
        raise PcapError(_pcap_ffi.string(err_buf).decode())

    net = net[0]
    mask = mask[0]

    promisc = int(bool(promisc))

    pcap_handle = _pcap_lib.pcap_open_live(
            device.encode(), snaplen, promisc, to_ms, err_buf)
    if pcap_handle == _pcap_ffi.NULL:
        raise PcapError(_pcap_ffi.string(err_buf).decode())

    return Reader(pcap_handle, net, mask)

def open_offline(filename):
    """ Returns a capture class `Reader` for a given filename.

    Internally performs `pcap_open_offline` and uses that to
    get the `Reader` object.
    Args:
        filename: string - Filename to open as `savefile`

    Returns:
        `Reader`

    Raises:
        `PcapError`
    """
    err_buf = _pcap_ffi.new('char [256]')
    pcap_handle = _pcap_lib.pcap_open_offline(filename.encode(), err_buf)
    if pcap_handle == _pcap_ffi.NULL:
        raise PcapError(_pcap_ffi.string(err_buf).decode())

    return Reader(pcap_handle)


def lookupdev():
    """Returns a device suitable for packet capture.

    This function is actually deprecated, but only supported for the
    existing users of `lookupdev`. Internally calls `findalldevs` and
    returns the name of the first device returned by that call.
    """

    message = "`lookupdev` is deprecated. Use `findalldevs` and the first "\
            "device returned by `findalldevs`."
    warnings.warn(message, DeprecationWarning, stacklevel=2)

    interfaces = findalldevs()
    if len(interfaces) > 0:
        return interfaces[0]
    else:
        return None


def findalldevs():
    """ Return's a 'list' of names of devices for packet capture.

    This function is a wrapeprover pcap_findalldevs.
    """

    err_buf = _pcap_ffi.new('char [256]')
    interfaces = _pcap_ffi.new('pcap_if_t **')
    result = _pcap_lib.pcap_findalldevs(interfaces, err_buf)
    if result != 0:
        raise PcapError(_pcap_ffi.string(err_buf).decode())

    interface_names = []
    iface = interfaces[0][0]
    while iface != _pcap_ffi.NULL:
        name = _pcap_ffi.string(iface.name).decode()
        interface_names.append(name)
        iface = iface.next

    _pcap_lib.pcap_freealldevs(interfaces[0])

    return interface_names

def compile():
    pass


def create():
    pass


class BPFProgram:
    pass


class Reader:
    """Reader Class (internal)

    This class performs all the work. This class is obtained typically by
    calling `open_live`. Takes `pcap_handle`  and net / mask as parameters.
    """

    def __init__(self, pcap_handle, net=0, mask=0):
        """Args:
            pcap_handle: `pcap_t *` handle.
            net :        network address
            mask:        netmask
        """
        self.__pcap_handle = pcap_handle
        self.__net = net
        self.__mask = mask

    def __del__(self):
        """Cleanup when the object is deleted.
        """
        self._do_close()

    def _do_close(self):
        """Cleans up internal handle.
        """
        try:
            pcap_close(self.__pcap_handle)
        except:
            pass
        finally:
            self.__pcap_handle = None

    def _raise_if_closed(self):
        """internal function that raises `PcapError` if the `__pcap_handle`
        is closed or is None.
        """

        if self.__pcap_handle is None:
            raise ValueError("Pcap Handle is already closed.")


    def activate(self):
        """Activate a capture handle created using `create`

        This function is usually not required to be called if we get the handle
        to the `Reader` object through `open_live` because, the `__pcap_handle`
        is then already activated

        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_activate(self.__pcap_handle)

        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        # FIXME: Also handle warnings
        return result

    def close(self):
        """Closes the internal PCAP handle, raising exception if already
        closed.
        """
        self._raise_if_closed()
        self._do_close()

    def datalink(self):
        """Returns data-link Layer Type for the pcap.

        Also Raises `PcapError` if  __pcap_handle is closed.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_datalink(self.__pcap_handle)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def dispatch(self, count, callback, user_data=_pcap_ffi.NULL):
        """Dispatches a callback function using `pcap_dispatch`

        `cffi` library takes care of releasing the GIL, so we are not required
        to worry about that we can simply call the func.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_dispatch(self.__pcap_handle, count,
                callback, user_data)
        if result < 0:
            if result == _pcap_lib.PCAP_ERROR_BREAK:
                # We are returning because someone called `pcap_breakloop`
                return 0
            else:
                result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
                raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def dump_open(self, filename):
        """Opens a given `filename` as `savefile`.
        """
        dump_handle = _pcap_lib.pcap_dump_open(self.__pcap_handle, filename.encode())

        if dump_handle == _pcap_ffi.NULL:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return _Dumper(dump_handle)

    def get_fd(self):
        """Returns the file descriptor corresponding to internal handle.
        """
        self._raise_if_closed()

        if sys.platform == 'win32':
            raise PcapError("This operation is not supported on 'Windows'.")

        return _pcap_lib.pcap_get_selectable_fd(self.__pcap_handle)

    def getmask(self):
        """Returns the netmask corresponding to interface for which `handle`
        is opened.
        """
        return str(ipaddress.IPv4Address(self.__mask))

    def getnet(self):
        """Returns the network corresponding to interface for which `handle`
        is opened.
        """
        return str(ipaddress.IPv4Address(self.__net))

    def getnonblock(self):
        """Returns the current non-blocking mode of the internal handle.
        Raises an error if the corresponding `pcap` call fails.
        """
        err_buf = _pcap_ffi.new('char [256]')
        result = _pcap_lib.pcap_getnonblock(self.__pcap_handle, err_buf)
        if result != 0:
            raise PcapError(_pcap_ffi.string(err_buf).decode())

        return result

    def loop(self):
        """Loops through internal pcap handle. Calls the callback. See
        `dispatch` above.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_loop(self.__pcap_handle, count,
                callback, user_data)
        if result < 0:
            if result == _pcap_lib.PCAP_ERROR_BREAK:
                # We are returning because someone called `pcap_breakloop`
                return 0
            else:
                result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
                raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def next(self):
        """Returns the next packet data as `Pkthdr` and `buffer`
        """
        self._raise_if_closed()

        pkthdr_ptr = _pcap_ffi.new('struct pcap_pkthdr **')
        pktbuf_ptr = _pcap_ffi.new('unsigned char **')

        result = _pcap_lib.pcap_next_ex(self.__pcap_handle, pkthdr_ptr,
                pktbuf_ptr)
        if result <= 0:
            if result in [0, _pcap_lib.PCAP_ERROR_BREAK]:
                return None, b''
            else:
                result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
                raise PcapError(_pcap_ffi.string(result_str).decode())

        pkthdr_ptr0 = pkthdr_ptr[0]
        caplen = pkthdr_ptr0.caplen

        pkt_data = _pcap_ffi.string(pktbuf_ptr[0])
        pkt_hdr = Pkthdr(pkthdr_ptr0)

        return pkt_hdr, pkt_data


    def sendpacket(self, data, length):
        """Sends the given data on the given PCAP Handle.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_sendpacket(self.__pcap_handle, data, length)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def set_buffer_size(self, buffersz):
        """Set's the pcap Buffer size to `buffersz`.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_set_buffer_size(self.__pcap_handle, buffersz)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def set_promisc(self, promisc):
        """Set's the promiscuous mode.
        """
        self._raise_if_closed()

        promisc = int(bool(promisc))
        result = _pcap_lib.pcap_set_promisc(self.__pcap_handle, promisc)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def set_rfmon(self, rfmon):
        """Set's the rf monitor mode.
        """
        self._raise_if_closed()

        rfmon = int(bool(rfmon))
        result = _pcap_lib.pcap_set_rfmon(self.__pcap_handle, rfmon)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def set_snaplen(self, snaplen):
        """Set's the snaplen on captured packets.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_set_snaplen(self.__pcap_handle, snaplen)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def set_timeout(self, to_ms):
        """Set's the snaplen on captured packets.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_set_timeout(self.__pcap_handle, to_ms)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def setdirection(self, direction):
        """Set's the snaplen on captured packets.
        """
        self._raise_if_closed()

        result = _pcap_lib.pcap_setdirection(self.__pcap_handle, direction)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return result

    def setfilter(self):
        raise NotImplementedError("Not implemented yet")

    def setnonblock(self, non_blocking):
        """Set's the current non-blocking mode of the internal handle.
        Raises an error if the corresponding `pcap` call fails.
        """
        non_blocking = int(bool(non_blocking))
        err_buf = _pcap_ffi.new('char [256]')
        result = _pcap_lib.pcap_setnonblock(self.__pcap_handle,
                non_blocking, err_buf)
        if result != 0:
            raise PcapError(_pcap_ffi.string(err_buf).decode())

        return result

    #FIXME: Add set_ts_precision and get_ts_precision and list_ts_precisions funcs.
    def stats(self):
        """Get's the PCAP Stats.
        """
        stats = _pcap_ffi.new('struct pcap_stat *')
        result = _pcap_lib.pcap_setdirection(self.__pcap_handle, direction)
        if result < 0:
            result_str = _pcap_lib.pcap_geterr(self.__pcap_handle)
            raise PcapError(_pcap_ffi.string(result_str).decode())

        return stats[0].ps_recv, stats[0].ps_drop, stats[0].ps_ifdrop

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):

        self._do_close()
        return False


class Pkthdr:
    """ Pkthdr class, wrapper over the `pcap_pkthdr` Struct

        This class is defined for using with Dumper and other classes,
        actually, eventually we'll support directly using the
        cdata structure itself.
    """

    def __init__(self, pcaphdr):
        """ Constructor:
            Args:
                pcaphdr: '<cdata struct pcap_pkthdr *>' object.
        """
        self.__ts = pcaphdr[0].ts
        self.__caplen = pcaphdr[0].caplen
        self.__len = pcaphdr[0].len

    @property
    def ts(self):
        return self.__ts

    @property
    def caplen(self):
        return self.__caplen

    @property
    def len(self):
        return self.__len

    def getts(self):
        return self.__ts.tv_sec, self.__ts.tv_usec

    def getcaplen(self):
        return self.__caplen

    def getlen(self):
        return self.__len


class _Dumper:
    """ Internal class Dumper. 'pcapy' does not provide an API to directly
        use this class. We may provide it.
    """
    _pcap_handle = None

    def __init__(self, pdumper):
        """ Constructor:

            Args:
                pdumper: ctype object for libpcap's `pcap_dumper_t`. The
                         constructor should not be called directly by the
                         application, instead should use the below
                         `from_filename` classmethod, to get the object.

        """
        self.__dumper = pdumper

    def __del__(self):
        """Destructor: Making sure we close our handle.
        """
        self._do_close()

    def _do_close(self):
        if self.__dumper:
            try:
                _pcap_lib.pcap_dump_close(self.__dumper)
            except:
                ## Log warning
                pass
            finally:
                self.__dumper = None


    @classmethod
    def from_filename(cls, filename, pcap_handle):
        """ Constructs a 'Dumper' object from the filename.

            Args:
                filename :    File name to be used for creating the dumper
                              object.
                pcap_handle : Should be opened by caller and passed to us.
                              This is also required to ensure that the
                              lifetime of the pcap_handle is greater than
                              ours.
        """
        dumper = _pcap_lib.pcap_dump_open(pcap_handle, filename.encode())
        return cls(dumper)

    def dump(self, hdr, data):
        """ Dumps into the file.

            Args:
                hdr: type: Pkthdr
                data: type: bytes
        """
        if not isinstance(hdr, Pkthdr):
            raise PcapError("Argument 'hdr' should be of type 'Pkthdr'")

        if not isinstance(data, bytes):
            raise PcapError("Argument 'data' shouldbe of type 'bytes'")

        pcap_pkthdr = _pcap_ffi.new('struct pcap_pkthdr *')
        pcap_pkthdr[0].ts = hdr.ts
        pcap_pkthdr[0].caplen = hdr.caplen
        pcap_pkthdr[0].len = hdr.len

        user = _pcap_ffi.cast('unsigned char *', self.__dumper)
        _pcap_lib.pcap_dump(user, pcap_pkthdr, data)

    def close(self):
        """Closes internal file object calling `pcap_dump_close`
        """
        self._do_close()


if __name__ == '__main__':

    h = _pcap_ffi.new('struct pcap_pkthdr *')
    ph = Pkthdr(h)
    print(ph.getts())
    print(ph.getlen())
    print(ph.getcaplen())

    print(lookupdev())
    print(findalldevs())
    r = open_live(lookupdev(), 0, 0, 10)
    print(r)
    print(r.set_promisc(1))
    print(r.close())
    #print(r.datalink())



from wishpy.wireshark.src.wireshark2.epan.epan import (
        epan_ffi,
        _sources,
        _extra_compile_args)

if __name__ == '__main__':
    epan_lib = epan_ffi.verify(
            _sources,
            #libraries=_libraries,
            extra_compile_args=_extra_compile_args,
            extra_link_args=_extra_link_args)

    print(epan_lib)
    print(epan_ffi.new('wtap_rec *'))



epan_address_h_types_cdef = """
/* address.h
 * Definitions for structures storing addresses, and for the type of
 * variables holding port-type values
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* Types of "global" addresses Wireshark knows about. */
/* Address types can be added here if there are many dissectors that use them or just
 * within a specific dissector.
 * If an address type is added here, it must be "registered" within address_types.c
 * For dissector address types, just use the address_type_dissector_register function
 * from address_types.h
 */
typedef enum {
    AT_NONE,               /* no link-layer address */
    AT_ETHER,              /* MAC (Ethernet, 802.x, FDDI) address */
    AT_IPv4,               /* IPv4 */
    AT_IPv6,               /* IPv6 */
    AT_IPX,                /* IPX */
    AT_FC,                 /* Fibre Channel */
    AT_FCWWN,              /* Fibre Channel WWN */
    AT_STRINGZ,            /* null-terminated string */
    AT_EUI64,              /* IEEE EUI-64 */
    AT_IB,                 /* Infiniband GID/LID */
    AT_AX25,               /* AX.25 */
    AT_VINES,              /* Banyan Vines address */

    AT_END_OF_LIST         /* Must be last in list */
} address_type;

typedef struct _address {
    int           type;         /* type of address */
    int           len;          /* length of address, in bytes */
    const void   *data;         /* pointer to address data */

    /* private */
    void         *priv;
} address;

/* Types of port numbers Wireshark knows about. */
typedef enum {
    PT_NONE,            /* no port number */
    PT_SCTP,            /* SCTP */
    PT_TCP,             /* TCP */
    PT_UDP,             /* UDP */
    PT_DCCP,            /* DCCP */
    PT_IPX,             /* IPX sockets */
    PT_DDP,             /* DDP AppleTalk connection */
    PT_IDP,             /* XNS IDP sockets */
    PT_USB,             /* USB endpoint 0xffff means the host */
    PT_I2C,
    PT_IBQP,            /* Infiniband QP number */
    PT_BLUETOOTH
} port_type;

"""



import sys
from datetime import datetime as dt
import codecs
import binascii
from queue import Queue
import threading

MAX_COUNT = -1

from wishpy.libpcap.lib.capturer import LibpcapCapturer
from wishpy.wireshark.lib.dissector import (
        WishpyDissectorQueuePython,
        setup_process,
        cleanup_process)

then = dt.now()

packet_queue = Queue()
c = LibpcapCapturer('wlp2s0', packet_queue)
c.open()

capture_thread = threading.Thread(target=c.start, args=(MAX_COUNT,))
capture_thread.start()


setup_process()

dissector = WishpyDissectorQueuePython(packet_queue)


try:

    for _, _, dissected in dissector.run(count=0):
        print(dissected)

except KeyboardInterrupt:
    print("User interrupted.")

except Exception as e:
    print(e)

finally:
    cleanup_process()

c.stop()
now = dt.now()

capture_thread.join()

#print(now - then)



epan_ftypes_h_types_cdef = """

/* ftypes.h
 * Definitions for field types
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 2001 Gerald Combs
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */



/* field types */
enum ftenum {
	FT_NONE,	/* used for text labels with no value */
	FT_PROTOCOL,
	FT_BOOLEAN,	/* TRUE and FALSE come from <glib.h> */
	FT_CHAR,	/* 1-octet character as 0-255 */
	FT_UINT8,
	FT_UINT16,
	FT_UINT24,	/* really a UINT32, but displayed as 6 hex-digits if FD_HEX*/
	FT_UINT32,
	FT_UINT40,	/* really a UINT64, but displayed as 10 hex-digits if FD_HEX*/
	FT_UINT48,	/* really a UINT64, but displayed as 12 hex-digits if FD_HEX*/
	FT_UINT56,	/* really a UINT64, but displayed as 14 hex-digits if FD_HEX*/
	FT_UINT64,
	FT_INT8,
	FT_INT16,
	FT_INT24,	/* same as for UINT24 */
	FT_INT32,
	FT_INT40, /* same as for UINT40 */
	FT_INT48, /* same as for UINT48 */
	FT_INT56, /* same as for UINT56 */
	FT_INT64,
	FT_IEEE_11073_SFLOAT,
	FT_IEEE_11073_FLOAT,
	FT_FLOAT,
	FT_DOUBLE,
	FT_ABSOLUTE_TIME,
	FT_RELATIVE_TIME,
	FT_STRING,
	FT_STRINGZ,	/* for use with proto_tree_add_item() */
	FT_UINT_STRING,	/* for use with proto_tree_add_item() */
	FT_ETHER,
	FT_BYTES,
	FT_UINT_BYTES,
	FT_IPv4,
	FT_IPv6,
	FT_IPXNET,
	FT_FRAMENUM,	/* a UINT32, but if selected lets you go to frame with that number */
	FT_PCRE,	/* a compiled Perl-Compatible Regular Expression object */
	FT_GUID,	/* GUID, UUID */
	FT_OID,		/* OBJECT IDENTIFIER */
	FT_EUI64,
	FT_AX25,
	FT_VINES,
	FT_REL_OID,	/* RELATIVE-OID */
	FT_SYSTEM_ID,
	FT_STRINGZPAD,	/* for use with proto_tree_add_item() */
	FT_FCWWN,
	FT_NUM_TYPES /* last item number plus one */
};

/* field types lengths */
#define FT_ETHER_LEN		6
#define FT_GUID_LEN		16
#define FT_IPv4_LEN		4
#define FT_IPv6_LEN		16
#define FT_IPXNET_LEN		4
#define FT_EUI64_LEN		8
#define FT_AX25_ADDR_LEN	7
#define FT_VINES_ADDR_LEN	6
#define FT_FCWWN_LEN		8
#define FT_VARINT_MAX_LEN	10	/* Because 64 / 7 = 9 and 64 % 7 = 1, get an uint64 varint need reads up to 10 bytes. */

typedef enum ftenum ftenum_t;

enum ft_framenum_type {
	FT_FRAMENUM_NONE,
	FT_FRAMENUM_REQUEST,
	FT_FRAMENUM_RESPONSE,
	FT_FRAMENUM_ACK,
	FT_FRAMENUM_DUP_ACK,
	FT_FRAMENUM_RETRANS_PREV,
	FT_FRAMENUM_RETRANS_NEXT,
	FT_FRAMENUM_NUM_TYPES /* last item number plus one */
};

typedef enum ft_framenum_type ft_framenum_type_t;

struct _ftype_t;
typedef struct _ftype_t ftype_t;

/* String representation types. */
enum ftrepr {
	FTREPR_DISPLAY,
	FTREPR_DFILTER
};

typedef enum ftrepr ftrepr_t;

// From <wireshark/epan/ipv4.h>
typedef struct {
	guint32	addr;	/* stored in host order */
	guint32	nmask;	/* stored in host order */
} ipv4_addr_and_mask;

// From <wireshark/epan/ipv6.h>
typedef struct {
	ws_in6_addr addr;
	guint32 prefix;
} ipv6_addr_and_prefix;

typedef struct _protocol_value_t
{
	tvbuff_t	*tvb;
	gchar		*proto_string;
} protocol_value_t;

typedef struct _fvalue_t {
	ftype_t	*ftype;
	union {
		/* Put a few basic types in here */
		guint32			uinteger;
		gint32			sinteger;
		guint64			integer64;
		guint64			uinteger64;
		gint64			sinteger64;
		gdouble			floating;
		gchar			*string;
		guchar			*ustring;
		GByteArray		*bytes;
		ipv4_addr_and_mask	ipv4;
		ipv6_addr_and_prefix	ipv6;
		e_guid_t		guid;
		nstime_t		time;
		protocol_value_t 	protocol;
		GRegex			*re;
		guint16			sfloat_ieee_11073;
		guint32			float_ieee_11073;
	} value;

	/* The following is provided for private use
	 * by the fvalue. */
	gboolean	fvalue_gboolean1;

} fvalue_t;

"""
_ = """

#ifndef __FTYPES_H__
#define __FTYPES_H__

#include <glib.h>
#include "../wmem/wmem.h"
#include "ws_symbol_export.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* Initialize the ftypes subsystem. Called once. */
void
ftypes_initialize(void);

/* ---------------- FTYPE ----------------- */

/* given two types, are they similar - for example can two
 * duplicate fields be registered of these two types. */
gboolean
ftype_similar_types(const enum ftenum ftype_a, const enum ftenum ftype_b);

/* Return a string representing the name of the type */
WS_DLL_PUBLIC
const char*
ftype_name(ftenum_t ftype);

/* Return a string presenting a "pretty" representation of the
 * name of the type. The pretty name means more to the user than
 * that "FT_*" name. */
WS_DLL_PUBLIC
const char*
ftype_pretty_name(ftenum_t ftype);

/* Returns length of field in packet, or 0 if not determinable/defined. */
int
ftype_length(ftenum_t ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_slice(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_eq(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_ne(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_gt(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_ge(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_lt(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_le(enum ftenum ftype);

gboolean
ftype_can_bitwise_and(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_contains(enum ftenum ftype);

WS_DLL_PUBLIC
gboolean
ftype_can_matches(enum ftenum ftype);

/* ---------------- FVALUE ----------------- */

#include <epan/ipv4.h>
#include <epan/ipv6.h>
#include <epan/guid-utils.h>

#include <epan/tvbuff.h>
#include <wsutil/nstime.h>
#include <epan/dfilter/drange.h>

fvalue_t*
fvalue_new(ftenum_t ftype);

void
fvalue_init(fvalue_t *fv, ftenum_t ftype);

WS_DLL_PUBLIC
fvalue_t*
fvalue_from_unparsed(ftenum_t ftype, const char *s, gboolean allow_partial_value, gchar **err_msg);

fvalue_t*
fvalue_from_string(ftenum_t ftype, const char *s, gchar **err_msg);

/* Returns the length of the string required to hold the
 * string representation of the the field value.
 *
 * Returns -1 if the string cannot be represented in the given rtype.
 *
 * The length DOES NOT include the terminating NUL. */
WS_DLL_PUBLIC
int
fvalue_string_repr_len(fvalue_t *fv, ftrepr_t rtype, int field_display);

/* Creates the string representation of the field value.
 * Memory for the buffer is allocated based on wmem allocator
 * provided.
 *
 * field_display parameter should be a BASE_ value (enum field_display_e)
 * BASE_NONE should be used if field information isn't available.
 *
 * Returns NULL if the string cannot be represented in the given rtype.*/
WS_DLL_PUBLIC char *
fvalue_to_string_repr(wmem_allocator_t *scope, fvalue_t *fv, ftrepr_t rtype, int field_display);

WS_DLL_PUBLIC ftenum_t
fvalue_type_ftenum(fvalue_t *fv);

const char*
fvalue_type_name(fvalue_t *fv);

void
fvalue_set_byte_array(fvalue_t *fv, GByteArray *value);

void
fvalue_set_bytes(fvalue_t *fv, const guint8 *value);

void
fvalue_set_guid(fvalue_t *fv, const e_guid_t *value);

void
fvalue_set_time(fvalue_t *fv, const nstime_t *value);

void
fvalue_set_string(fvalue_t *fv, const gchar *value);

void
fvalue_set_protocol(fvalue_t *fv, tvbuff_t *value, const gchar *name);

void
fvalue_set_uinteger(fvalue_t *fv, guint32 value);

void
fvalue_set_sinteger(fvalue_t *fv, gint32 value);

void
fvalue_set_uinteger64(fvalue_t *fv, guint64 value);

void
fvalue_set_sinteger64(fvalue_t *fv, gint64 value);

void
fvalue_set_floating(fvalue_t *fv, gdouble value);

WS_DLL_PUBLIC
gpointer
fvalue_get(fvalue_t *fv);

WS_DLL_PUBLIC guint32
fvalue_get_uinteger(fvalue_t *fv);

WS_DLL_PUBLIC gint32
fvalue_get_sinteger(fvalue_t *fv);

WS_DLL_PUBLIC
guint64
fvalue_get_uinteger64(fvalue_t *fv);

WS_DLL_PUBLIC
gint64
fvalue_get_sinteger64(fvalue_t *fv);

WS_DLL_PUBLIC double
fvalue_get_floating(fvalue_t *fv);

gboolean
fvalue_eq(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_ne(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_gt(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_ge(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_lt(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_le(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_bitwise_and(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_contains(const fvalue_t *a, const fvalue_t *b);

gboolean
fvalue_matches(const fvalue_t *a, const fvalue_t *b);

guint
fvalue_length(fvalue_t *fv);

fvalue_t*
fvalue_slice(fvalue_t *fv, drange_t *dr);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __FTYPES_H__ */

/*
 * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
 *
 * Local variables:
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: t
 * End:
 *
 * vi: set shiftwidth=8 tabstop=8 noexpandtab:
 * :indentSize=8:tabSize=8:noTabs=false:
 */
"""



epan_guid_utils_h_types_cdef = """

/* guid-utils.h
 * Definitions for GUID handling
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 *
 * Copyright 1998 Gerald Combs
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#define GUID_LEN	16

/* Note: this might be larger than GUID_LEN, so don't overlay data in packets
   with this. */
typedef struct _e_guid_t {
    guint32 data1;
    guint16 data2;
    guint16 data3;
    guint8  data4[8];
} e_guid_t;

"""

epan_guid_utils_h_funcs_cdef = """

extern void guids_init(void);

/* add a GUID */
extern void guids_add_guid(const e_guid_t *guid, const gchar *name);

/* try to get registered name for this GUID */
extern const gchar *guids_get_guid_name(const e_guid_t *guid);

/* resolve GUID to name (or if unknown to hex string) */
/* (if you need hex string only, use guid_to_str instead) */
extern const gchar* guids_resolve_guid_to_str(const e_guid_t *guid);

/* add a UUID (dcerpc_init_uuid() will call this too) */
#define guids_add_uuid(uuid, name) guids_add_guid((const e_guid_t *) (uuid), (name))

/* try to get registered name for this UUID */
#define guids_get_uuid_name(uuid) guids_get_guid_name((e_guid_t *) (uuid))

/* resolve UUID to name (or if unknown to hex string) */
/* (if you need hex string only, use guid_to_str instead) */
#define guids_resolve_uuid_to_str(uuid) guids_resolve_guid_to_str((e_guid_t *) (uuid))

extern int guid_cmp(const e_guid_t *g1, const e_guid_t *g2);

#endif /* __GUID_UTILS_H__ */
"""
